@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from __future__ import print_function
 
 # plotting
@@ -6,6 +7,7 @@ from matplotlib import pyplot as plt
 
 from lifelines.utils import coalesce
 
+
 def plot_lifetimes(lifetimes, event_observed=None, birthtimes=None, order=False):
     """
     Parameters:
@@ -86,7 +88,7 @@ def plot_regressions(self):
             y = get_loc(self.cumulative_hazards_[column]).values
             y_upper = get_loc(self.confidence_intervals_[column].ix['upper']).values
             y_lower = get_loc(self.confidence_intervals_[column].ix['lower']).values
-            shaded_plot(x, y, y_upper, y_lower, ax=kwargs["ax"], label=coalesce(kwargs.get('label'),column))
+            shaded_plot(x, y, y_upper, y_lower, ax=kwargs["ax"], label=coalesce(kwargs.get('label'), column))
 
         if legend:
             kwargs["ax"].legend()
@@ -94,16 +96,17 @@ def plot_regressions(self):
         return kwargs["ax"]
     return plot
 
+
 def plot_estimate(self, estimate):
     doc_string = """"
-        Plots a pretty version of the fitted %s. 
-        
+        Plots a pretty version of the fitted %s.
+
         Matplotlib plot arguments can be passed in inside the kwargs, plus
 
         Parameters:
           flat: an opiniated design style with stepped lines and no shading. Similar to R's plotting. Default: False
           show_censors: place markers at censorship events. Default: False
-          censor_styles: If show_censors, this dictionary will be passed into the plot call. 
+          censor_styles: If show_censors, this dictionary will be passed into the plot call.
           ci_alpha: the transparency level of the confidence interval. Default: 0.3
           ci_force_lines: force the confidence intervals to be line plots (versus default shaded areas). Default: False
           ci_show=True: show confidence intervals. Default: True
@@ -117,7 +120,7 @@ def plot_estimate(self, estimate):
 
         Returns:
           ax: a pyplot axis object
-        """%estimate
+        """ % estimate
 
     def plot(ix=None, iloc=None, flat=False, show_censors=False, censor_styles={},
              ci_legend=False, ci_force_lines=False, ci_alpha=0.25, ci_show=True,
@@ -127,8 +130,8 @@ def plot_estimate(self, estimate):
 
         if "ax" not in kwargs:
             kwargs["ax"] = plt.figure().add_subplot(111)
-        kwargs['color'] = coalesce( kwargs.get('c'), kwargs.get('color'), next(kwargs["ax"]._get_lines.color_cycle) )
-        kwargs['drawstyle'] = coalesce( kwargs.get('drawstyle'), 'steps-post')
+        kwargs['color'] = coalesce(kwargs.get('c'), kwargs.get('color'), next(kwargs["ax"]._get_lines.color_cycle))
+        kwargs['drawstyle'] = coalesce(kwargs.get('drawstyle'), 'steps-post')
 
         # R-style graphics
         if flat:
@@ -154,21 +157,21 @@ def plot_estimate(self, estimate):
 
         # plot censors
         if show_censors and self.event_table['censored'].sum() > 0:
-            cs = {'marker':'+','ms':12, 'mew':1}
+            cs = {'marker': '+', 'ms': 12, 'mew': 1}
             cs.update(censor_styles)
             times = get_loc(self.event_table.ix[(self.event_table['censored'] > 0)]).index.values.astype(float)
             v = self.predict(times)
-            kwargs['ax'].plot(times, v, linestyle='None', color=kwargs['color'], **cs )
-                                        
+            kwargs['ax'].plot(times, v, linestyle='None', color=kwargs['color'], **cs)
+
         # plot esimate
         get_loc(estimate_).plot(**kwargs)
 
         # plot confidence intervals
         if ci_show:
             if ci_force_lines:
-               get_loc(confidence_interval_).plot(linestyle="-", linewidth=1,
+                get_loc(confidence_interval_).plot(linestyle="-", linewidth=1,
                                                    c=kwargs['color'], legend=True,
-                                                   drawstyle=kwargs.get('drawstyle', 'default'), 
+                                                   drawstyle=kwargs.get('drawstyle', 'default'),
                                                    ax=kwargs['ax'], alpha=0.6)
             else:
                 x = get_loc(confidence_interval_).index.values.astype(float)
@@ -184,10 +187,10 @@ def plot_estimate(self, estimate):
 def fill_between_steps(x, y1, y2=0, h_align='left', ax=None, **kwargs):
     ''' Fills a hole in matplotlib: Fill_between for step plots.
     https://gist.github.com/thriveth/8352565
- 
+
     Parameters :
     ------------
- 
+
     x : array-like
         Array/vector of index values. These are assumed to be equally-spaced.
         If not, the result will probably look weird...
@@ -195,9 +198,9 @@ def fill_between_steps(x, y1, y2=0, h_align='left', ax=None, **kwargs):
         Array/vector of values to be filled under.
     y2 : array-Like
         Array/vector or bottom values for filled area. Default is 0.
- 
+
     **kwargs will be passed to the matplotlib fill_between() function.
- 
+
     '''
     # If no Axes opject given, grab the current one:
     if ax is None:
@@ -208,19 +211,19 @@ def fill_between_steps(x, y1, y2=0, h_align='left', ax=None, **kwargs):
     xstep = (x[1:] - x[:-1]).mean()
     # Now: add one step at end of row.
     xx = np.append(xx, xx.max() + xstep)
- 
+
     # Make it possible to change step alignment.
     if h_align == 'mid':
         xx -= xstep / 2.
     elif h_align == 'right':
         xx -= xstep
- 
+
     # Also, duplicate each y coordinate in both arrays
     y1 = y1.repeat(2)
-    if type(y2) == np.ndarray:
+    if isinstance(y2, np.ndarray):
         y2 = y2.repeat(2)
- 
+
     # now to the plotting part:
     ax.fill_between(xx, y1, y2=y2, **kwargs)
- 
+
     return ax
